package net.sf.ejp.tools.formatonsave.core;

import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitEditor;
import org.eclipse.jdt.internal.ui.javaeditor.CompilationUnitHelper;
import org.eclipse.jdt.internal.ui.javaeditor.ISavePolicy;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;

public class EditorPatcherThread extends Thread
{
  private boolean stopRequested = false;

  public EditorPatcherThread()
  {
    super("Editor-Patcher");
    setPriority(MIN_PRIORITY);
  }

  public synchronized void stopAsap()
  {
    stopRequested = true;
  }

  private void patch() throws PatchingFailedException
  {
    IWorkbenchWindow window = WorkbenchHelper.getWorkbenchWindow();
    if (window == null)
    {
      Logger.log("window == null");
      return;
    }
    IWorkbenchPage page = window.getActivePage();
    if (page == null)
    {
      Logger.log("page == null");
      return;
    }
    IEditorPart part = page.getActiveEditor();
    if (part == null)
    {
      Logger.log("part == null");
      return;
    }

    if (!(part instanceof CompilationUnitEditor))
    {
      Logger.log("part not a CompilationUnitEditor");
      return;
    }

    CompilationUnitEditor editor = (CompilationUnitEditor) part;
    ISavePolicy savePolicy = getSavePolicy(editor);
    if (savePolicy != null)
    {
      if (!(savePolicy instanceof EditorSavePolicy))
      {
        Logger.log("editor already patched: " + savePolicy.getClass());
        return;
      }
      // patch already installed
      return;
    }

    setSavePolicy(editor, new EditorSavePolicy());
    Logger.log("succesfully patched");
  }

  private ISavePolicy getSavePolicy(CompilationUnitEditor _editor) throws PatchingFailedException
  {
    ISavePolicy result;
    try
    {
      result = CompilationUnitHelper.getSavePolicy(_editor);
    }
    catch (Throwable t)
    {
      throw new PatchingFailedException(ErrorCodes.GET_SAVED_POLICY_FAILED,
          "Failed to get current save policy", t);
    }
    return result;
  }

  private void setSavePolicy(CompilationUnitEditor _editor, ISavePolicy _savePolicy)
      throws PatchingFailedException
  {
    try
    {
      CompilationUnitHelper.setSavePolicy(_editor, _savePolicy);
    }
    catch (Throwable t)
    {
      throw new PatchingFailedException(ErrorCodes.SET_SAVED_POLICY_FAILED,
          "Failed to set current save policy", t);
    }
  }

  private synchronized boolean isStopRequested()
  {
    return stopRequested;
  }

  public void run()
  {
    for (;;)
    {
      try
      {
        sleep(1000);
      }
      catch (InterruptedException e)
      {
        Logger.log(e.toString());
        break;
      }

      if (isStopRequested())
      {
        break;
      }

      try
      {
        patch();
      }
      catch (PatchingFailedException e)
      {
        Logger.logError(e.getErrorCode(), e.getMessage(), e.getCause());
      }
    }
  }

  private static class PatchingFailedException extends Exception
  {
    /**
     * Generated ID
     */
    private static final long serialVersionUID = 3257004354321003831L;

    private final int errorCode;

    public PatchingFailedException(int _errorCode, String _message, Throwable _cause)
    {
      super(_message, _cause);
      errorCode = _errorCode;
    }

    public int getErrorCode()
    {
      return errorCode;
    }
  }
}
