package net.sf.ejp.tools.formatonsave.core;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class FormatOnSavePlugin extends AbstractUIPlugin
{
  // The shared instance.
  private static FormatOnSavePlugin plugin;

  // Resource bundle.
  private ResourceBundle resourceBundle;

  private EditorPatcherThread editorPatcherThread = null;

  /**
   * The constructor.
   */
  public FormatOnSavePlugin()
  {
    super();
    plugin = this;
    try
    {
      resourceBundle = ResourceBundle
          .getBundle("net.sf.tools.formatonsave.FormatOnSavePluginResources");
    }
    catch (MissingResourceException x)
    {
      resourceBundle = null;
    }
  }

  /**
   * This method is called upon plug-in activation
   */
  public void start(BundleContext context) throws Exception
  {
    super.start(context);

    Logger.log("start");
    editorPatcherThread = new EditorPatcherThread();
    editorPatcherThread.start();
  }

  /**
   * This method is called when the plug-in is stopped
   */
  public void stop(BundleContext context) throws Exception
  {
    try
    {
      Logger.log("stop");
      editorPatcherThread.stopAsap();
      editorPatcherThread.join();
    }
    finally
    {
      super.stop(context);
    }
  }

  /**
   * Returns the shared instance.
   */
  public static FormatOnSavePlugin getDefault()
  {
    return plugin;
  }

  /**
   * Returns the string from the plugin's resource bundle, or 'key' if not
   * found.
   */
  public static String getResourceString(String key)
  {
    ResourceBundle bundle = FormatOnSavePlugin.getDefault().getResourceBundle();
    try
    {
      return (bundle != null) ? bundle.getString(key) : key;
    }
    catch (MissingResourceException e)
    {
      return key;
    }
  }

  /**
   * Returns the plugin's resource bundle,
   */
  public ResourceBundle getResourceBundle()
  {
    return resourceBundle;
  }
}
